import * as jotai from 'jotai';
import { createStore } from 'jotai';
import { CSSProperties } from 'react';
import { Root } from 'react-dom/client';
import { WalletContextState } from '@jup-ag/wallet-adapter';
import { PublicKey, TransactionError } from '@solana/web3.js';
import { SwapResult } from '@jup-ag/react-hook';
import JSBI from 'jsbi';
import * as superstruct from 'superstruct';
import { Infer } from 'superstruct';

declare const AGGREGATOR_SOURCES: {
    readonly METIS: "metis";
    readonly JUPITERZ: "jupiterz";
    readonly HASHFLOW: "hashflow";
    readonly DFLOW: "dflow";
};
type AggregatorSources = (typeof AGGREGATOR_SOURCES)[keyof typeof AGGREGATOR_SOURCES];
interface UltraQuoteResponse {
    inputMint: string;
    inAmount: string;
    outputMint: string;
    outAmount: string;
    otherAmountThreshold: string;
    priceImpactPct: string;
    routePlan: {
        swapInfo: {
            inputMint: string;
            inAmount: string;
            outputMint: string;
            outAmount: string;
            ammKey: string;
            label: string;
            feeAmount: JSBI;
            feeMint: string;
        };
        percent: number;
    }[];
    contextSlot: number;
    transaction: string | null;
    swapType: 'ultra';
    gasless: boolean;
    requestId: string;
    prioritizationFeeLamports?: number;
    feeBps: number;
    router: AggregatorSources;
}

declare const FormattedUltraQuoteResponse: superstruct.Struct<{
    inputMint: PublicKey;
    outputMint: PublicKey;
    inAmount: JSBI;
    outAmount: JSBI;
    otherAmountThreshold: JSBI;
    priceImpactPct: string;
    routePlan: {
        swapInfo: {
            label: string;
            inputMint: string;
            outputMint: string;
            ammKey: PublicKey;
            inAmount: JSBI;
            outAmount: JSBI;
            feeAmount: JSBI;
            feeMint: PublicKey;
        };
        percent: number;
    }[];
    slippageBps: number;
    contextSlot: number;
    transaction: string | null;
    swapType: string;
    gasless: boolean;
    requestId: string;
    feeBps: number;
    router: string;
    computedAutoSlippage?: number | undefined;
    prioritizationFeeLamports?: number | undefined;
}, {
    inputMint: superstruct.Struct<PublicKey, null>;
    inAmount: superstruct.Struct<JSBI, null>;
    outputMint: superstruct.Struct<PublicKey, null>;
    outAmount: superstruct.Struct<JSBI, null>;
    otherAmountThreshold: superstruct.Struct<JSBI, null>;
    priceImpactPct: superstruct.Struct<string, null>;
    routePlan: superstruct.Struct<{
        swapInfo: {
            label: string;
            inputMint: string;
            outputMint: string;
            ammKey: PublicKey;
            inAmount: JSBI;
            outAmount: JSBI;
            feeAmount: JSBI;
            feeMint: PublicKey;
        };
        percent: number;
    }[], superstruct.Struct<{
        swapInfo: {
            label: string;
            inputMint: string;
            outputMint: string;
            ammKey: PublicKey;
            inAmount: JSBI;
            outAmount: JSBI;
            feeAmount: JSBI;
            feeMint: PublicKey;
        };
        percent: number;
    }, {
        swapInfo: superstruct.Struct<{
            label: string;
            inputMint: string;
            outputMint: string;
            ammKey: PublicKey;
            inAmount: JSBI;
            outAmount: JSBI;
            feeAmount: JSBI;
            feeMint: PublicKey;
        }, {
            ammKey: superstruct.Struct<PublicKey, null>;
            label: superstruct.Struct<string, null>;
            inputMint: superstruct.Struct<string, null>;
            outputMint: superstruct.Struct<string, null>;
            inAmount: superstruct.Struct<JSBI, null>;
            outAmount: superstruct.Struct<JSBI, null>;
            feeAmount: superstruct.Struct<JSBI, null>;
            feeMint: superstruct.Struct<PublicKey, null>;
        }>;
        percent: superstruct.Struct<number, null>;
    }>>;
    slippageBps: superstruct.Struct<number, null>;
    contextSlot: superstruct.Struct<number, null>;
    computedAutoSlippage: superstruct.Struct<number | undefined, null>;
    transaction: superstruct.Struct<string | null, null>;
    swapType: superstruct.Struct<string, null>;
    gasless: superstruct.Struct<boolean, null>;
    requestId: superstruct.Struct<string, null>;
    prioritizationFeeLamports: superstruct.Struct<number | undefined, null>;
    feeBps: superstruct.Struct<number, null>;
    router: superstruct.Struct<string, null>;
}>;
type FormattedUltraQuoteResponse = Infer<typeof FormattedUltraQuoteResponse>;

type QuoteResponse = {
    original: UltraQuoteResponse;
    quoteResponse: FormattedUltraQuoteResponse;
};

declare global {
  interface Window {
    Jupiter: JupiterTerminal;
  }
}

/** The position of the widget */

type WidgetPosition = 'bottom-left' | 'bottom-right' | 'top-left' | 'top-right';
/** The size of the widget */
type WidgetSize = 'sm' | 'default';

type SwapMode = "ExactInOrOut" | "ExactIn" | "ExactOut";

interface FormProps {
  /** Default to `ExactInOrOut`. ExactOut can be used to get an exact output of a token (e.g. for Payments) */
  swapMode?: SwapMode;
  /** Initial amount to swap */
  initialAmount?: string;
  /** When true, user cannot change the amount (e.g. for Payments) */
  fixedAmount?: boolean;
  /** Initial input token to swap */
  initialInputMint?: string;
  /** Initial output token to swap */
  initialOutputMint?: string;
  /** Based token that can never changed  */
  fixedMint?: string;
  /** Referral account to use for the swap */
  referralAccount?: string;
  /** Referral fee to use for the swap */
  referralFee?: number;
}

/** Built in support for these explorers */
type DEFAULT_EXPLORER = 'Solana Explorer' | 'Solscan' | 'Solana Beach' | 'SolanaFM';

interface IInit {
  /** Settings saved in local storage will be prefixed with this
   * You can reset your user's local storage by changing this value
   */
  localStoragePrefix?: string;

  /** Configure Terminal's behaviour and allowed actions for your user */
  formProps?: FormProps;
  /** Default explorer for your user */
  defaultExplorer?: DEFAULT_EXPLORER;
  /** Auto connect to wallet on subsequent visits */
  autoConnect?: boolean;

  /** Display & Styling */
  /** Display mode */
  displayMode?: 'modal' | 'integrated' | 'widget';
  /** When displayMode is 'integrated', this is the id of the element to render the integrated widget into */
  integratedTargetId?: string;
  /** When displayMode is 'widget', this is the behaviour and style of the widget */
  widgetStyle?: {
    position?: WidgetPosition;
    size?: WidgetSize;
  };
  /** In case additional styling is needed for Terminal container */
  containerStyles?: CSSProperties;
  /** In case additional styling is needed for Terminal container */
  containerClassName?: string;

  /** When true, wallet connection are handled by your dApp, and use `syncProps()` to syncronise wallet state with Terminal */
  enableWalletPassthrough?: boolean;
  /** Optional, if wallet state is ready, you can pass it in here, or just use `syncProps()` */
  passthroughWalletContextState?: WalletContextState;
  /** When enableWalletPassthrough is true, this allows Terminal to callback your dApp's wallet connection flow */
  onRequestConnectWallet?: () => void | Promise<void>;

  /** Callbacks */
  /** When an error has occured during swap */
  onSwapError?: ({
    error,
    quoteResponseMeta,
  }: {
    error?: TransactionError;
    quoteResponseMeta: QuoteResponse | null;
  }) => void;
  /** When a swap has been successful */
  onSuccess?: ({
    txid,
    swapResult,
    quoteResponseMeta,
  }: {
    txid: string;
    swapResult: SwapResult;
    quoteResponseMeta: QuoteResponse | null;
  }) => void;
  /** Callback when there's changes to the form */
  onFormUpdate?: (form: IForm) => void;
  /** Callback when there's changes to the screen */
  onScreenUpdate?: (screen: IScreen) => void;

  /** Internal resolves */

  /** Internal use to resolve domain when loading script */
  scriptDomain?: string;
}

interface JupiterTerminal {
  _instance: JSX.Element | null;
  init: (props: IInit) => void;
  resume: () => void;
  close: () => void;
  root: Root | null;

  /** Passthrough */
  enableWalletPassthrough: boolean;
  onRequestConnectWallet: IInit['onRequestConnectWallet'];
  store: ReturnType<typeof createStore>;
  syncProps: (props: { passthroughWalletContextState?: IInit['passthroughWalletContextState'] }) => void;

  /** Callbacks */
  onSwapError: IInit['onSwapError'];
  onSuccess: IInit['onSuccess'];
  onFormUpdate: IInit['onFormUpdate'];
  onScreenUpdate: IInit['onScreenUpdate'];

  /** Special props */
  localStoragePrefix: string;
}

declare const appProps: jotai.PrimitiveAtom<IInit | undefined> & {
    init: IInit | undefined;
};
declare const resume: () => void;
declare const close: () => void;
declare const syncProps: (props: {
    enableWalletPassthrough?: IInit['enableWalletPassthrough'];
    passthroughWalletContextState?: IInit['passthroughWalletContextState'];
}) => void;

declare function init(props: IInit): Promise<void>;

export { appProps, close, init, resume, syncProps };
